unit WordAndOutlook_IMPL;

interface

uses
  SysUtils, ComObj, ComServ, ActiveX, Variants, Office2000, adxAddIn, WordAndOutlook_TLB,
  adxHostAppEvents, Word2000, Classes, StdVcl, Outlook2000;

type
  TCoWordAndOutlook = class(TadxAddin, ICoWordAndOutlook)
  end;

  TAddInModule = class(TadxCOMAddInModule)
    adxOlInspectorCommandBar1: TadxOlInspectorCommandBar;
    adxOutlookAppEvents1: TadxOutlookAppEvents;
    adxWordAppEvents1: TadxWordAppEvents;
    procedure adxOlInspectorCommandBar1Controls0Click(Sender: TObject);
    procedure adxWordAppEvents1WindowActivate(ASender: TObject;
      const Doc: _Document; const Wn: Window);
    procedure adxCOMAddInModuleAddInInitialize(Sender: TObject);
    procedure adxOlInspectorCommandBar1BeforeAdd(
      CmdBar: TadxOlInspectorCommandBar; Inspector: _Inspector;
      var Cancel: Boolean);
    procedure adxCOMAddInModuleBeforeAddinUnregister(Sender: TObject);
  private
    procedure DeleteControl(CmdBar: CommandBar; AdxControl: TadxCommandBarControl);
    function GetOfficeControl (CmdBar: CommandBar; AdxControl: TadxCommandBarControl):CommandBarControl;
    function GetMsoControlType(AdxControl: TadxCommandBarControl):MsoControlType;
  protected
  public
  end;

implementation

uses Windows, Dialogs, Registry;

{$R *.dfm}

procedure TAddInModule.adxCOMAddInModuleAddInInitialize(Sender: TObject);
begin
  FastCommandBarSearch := True;
end;

procedure TAddInModule.adxOlInspectorCommandBar1Controls0Click(Sender: TObject);
begin
  ShowMessage('Clicked: Outlook Inspector');  //
end;

procedure TAddInModule.adxWordAppEvents1WindowActivate(ASender: TObject;
  const Doc: _Document; const Wn: Window);
var
  CmdBar: CommandBar;
begin
  try
    // 'Custom 2' - Outlook Inspector
    CmdBar := Doc.CommandBars.Item['Custom 2'];
    if Assigned(CmdBar) then begin
      CmdBar.Set_Visible(Wn.EnvelopeVisible);
    end;
  except
    Windows.beep(600,600);
    // not found
  end;
end;

procedure TAddInModule.adxOlInspectorCommandBar1BeforeAdd(
  CmdBar: TadxOlInspectorCommandBar; Inspector: _Inspector;
  var Cancel: Boolean);
var
  R: TRegistry;
begin
  R := TRegistry.Create();
  R.RootKey := HKEY_CURRENT_USER;
  if R.OpenKey(Self.RegistryKey, False) then begin
    R.DeleteKey('Controls');
    R.DeleteKey('CommandBars');
  end;
  R.Free();
  if (Inspector.EditorType = olEditorWord) then
    CmdBar.Temporary := false
  else
    CmdBar.Temporary := true;
end;

procedure TAddInModule.adxCOMAddInModuleBeforeAddinUnregister(
  Sender: TObject);
var
  CmdBar: CommandBar;
  i: integer;
  word: TWordApplication;
begin
    word := TWordApplication.Create(nil);
    try
      try
        // 'Custom 2' - Outlook Inspector
        CmdBar := word.CommandBars.Item['Custom 2'];
      except
        Exit; // my commandbar doesn't exist in Word
      end;

      for i := adxOlInspectorCommandBar1.Controls.Count downto 1 do begin
        DeleteControl(CmdBar, adxOlInspectorCommandBar1.Controls[i-1]);
      end;
      CmdBar.Delete;
    finally
      word.Free;
    end;
end;

procedure TAddInModule.DeleteControl(CmdBar: CommandBar; AdxControl: TadxCommandBarControl);
var
  i: integer;
  OfficeControl: CommandBarControl;
begin
  OfficeControl := GetOfficeControl(CmdBAr, AdxControl);
  if OfficeControl <> nil then begin
    if AdxControl is TadxCommandBarPopup then begin
      for i := AdxControl.AsPopup.Controls.Count downto 1 do begin
        DeleteControl(CmdBar, AdxControl.AsPopup.Controls[i-1]);
      end;
    end;
    OfficeControl.Delete(false);
  end;
end;

function TAddInModule.GetOfficeControl(CmdBar: CommandBar;
  AdxControl: TadxCommandBarControl): CommandBarControl;
begin
  if (AdxControl.OfficeID <> 1) then
    Result := CmdBar.FindControl(GetMsoControlType(AdxControl), AdxControl.OfficeID, EmptyParam, EmptyParam, true)
  else
    Result := CmdBar.FindControl(GetMsoControlType(AdxControl), 1, AdxControl.OfficeTag, EmptyParam, true);
end;

function TAddInModule.GetMsoControlType(AdxControl: TadxCommandBarControl):MsoControlType;
begin
  if AdxControl.AsButton <> nil then Result := msoControlButton;
  if AdxControl.AsEdit <> nil then Result := msoControlComboBox;
  if AdxControl.AsComboBox <> nil then Result := msoControlComboBox;
  if AdxControl.AsDropdownList <> nil then Result := msoControlComboBox;
  if AdxControl.AsPopup <> nil then Result := msoControlPopup;
  if AdxControl.AsAdvancedControl <> nil then Result := msoControlButton;
end;

initialization
  TadxFactory.Create(ComServer, TCoWordAndOutlook, CLASS_CoWordAndOutlook, TAddInModule);

end.
